package com.gic.finance.service.outer.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.valid.ValidParamsUtils;
import com.gic.finance.constant.InvoiceUseStatusEnum;
import com.gic.finance.dto.InvoiceAccountDTO;
import com.gic.finance.entity.TabInvoiceAccount;
import com.gic.finance.service.InvoiceAccountApiService;
import com.gic.finance.service.InvoiceAccountService;

@Service("invoiceAccountApiService")
public class InvoiceAccountApiServiceImpl implements InvoiceAccountApiService {
    @Autowired
    private InvoiceAccountService invoiceAccountService;

    @Override
    public ServiceResponse<Void> save(InvoiceAccountDTO invoiceAccountDTO) {
        ServiceResponse paramValid = ValidParamsUtils.allCheckValidate(invoiceAccountDTO, InvoiceAccountDTO.Save.class);
        if (!paramValid.isSuccess()) {
            return paramValid;
        }
        //关闭之前的
        invoiceAccountService.closeStatus();
        invoiceAccountDTO.setCreateTime(new Date());
        invoiceAccountDTO.setUpdateTime(new Date());
        //启用默认
        invoiceAccountDTO.setStatus(InvoiceUseStatusEnum.ENABLE.getCode());
        invoiceAccountService.save(invoiceAccountDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> update(InvoiceAccountDTO invoiceAccountDTO) {
        ServiceResponse paramValid = ValidParamsUtils.allCheckValidate(invoiceAccountDTO, InvoiceAccountDTO.Save.class,
                InvoiceAccountDTO.Edit.class);
        if (!paramValid.isSuccess()) {
            return paramValid;
        }
        TabInvoiceAccount tab = invoiceAccountService.getById(invoiceAccountDTO.getInvoiceAccountId());
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键有误");
        }
        invoiceAccountService.update(invoiceAccountDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> enable(Integer id) {
        TabInvoiceAccount tab = invoiceAccountService.getById(id);
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键有误");
        }
        //关闭之前的
        invoiceAccountService.closeStatus();
        InvoiceAccountDTO dto = new InvoiceAccountDTO();
        dto.setInvoiceAccountId(id);
        dto.setStatus(InvoiceUseStatusEnum.ENABLE.getCode());
        invoiceAccountService.update(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> disable(Integer id) {
        TabInvoiceAccount tab = invoiceAccountService.getById(id);
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键有误");
        }
        TabInvoiceAccount enableTab = invoiceAccountService.getEnable();
        if (enableTab.getInvoiceAccountId().intValue() == id.intValue()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "不能关闭，必须有一个是开启状态");
        }
        InvoiceAccountDTO dto = new InvoiceAccountDTO();
        dto.setInvoiceAccountId(id);
        dto.setStatus(InvoiceUseStatusEnum.DISABLE.getCode());
        invoiceAccountService.update(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<InvoiceAccountDTO> getById(Integer invoiceAccountId) {
        TabInvoiceAccount tab = invoiceAccountService.getById(invoiceAccountId);
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键有误");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(InvoiceAccountDTO.class, tab));
    }

    @Override
    public ServiceResponse<InvoiceAccountDTO> getEnable() {
        TabInvoiceAccount tab = invoiceAccountService.getEnable();
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有启用的开票户信息，请启用");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(InvoiceAccountDTO.class, tab));
    }

    @Override
    public ServiceResponse<List<InvoiceAccountDTO>> listInvoiceAccount() {
        List<TabInvoiceAccount> accountList = invoiceAccountService.listInvoiceAccount();
        if (CollectionUtils.isNotEmpty(accountList)) {
            return ServiceResponse.success(EntityUtil.changeEntityListNew(InvoiceAccountDTO.class, accountList));
        }
        return ServiceResponse.success(new ArrayList<>());
    }
}
